'use strict';

const _ = require('lodash');
const domain = require('domain');
const cryptoRandomString = require('crypto-random-string');

const httpInfraSymbol = Symbol('http-infra');

function requireCorrelation() {
    if (!domain.active[httpInfraSymbol]) {
        throw new Error('Active domain isn\'t registered.');
    }
}

module.exports = {
    registerCorrelation(authenticatedEntity = null, requestId = null) {
        const activeDomain = domain.active;

        if (activeDomain[httpInfraSymbol]) {
            throw new Error('Can\'t register correlation. Domain is already registered');
        }

        if (!requestId) {
            requestId = cryptoRandomString({ length: 16, type: 'alphanumeric' });
        }

        activeDomain[httpInfraSymbol] = {
            authenticatedEntity,
            requestId
        };
    },

    extendCorrelationId(baseId) {
        const activeDomain = domain.active;
        _.set(
            activeDomain[httpInfraSymbol],
            'requestId',
            `${baseId}:${cryptoRandomString({ length: 16, type: 'alphanumeric' })}`
        );
    },

    getRequestId() {
        requireCorrelation();

        return domain.active[httpInfraSymbol].requestId;
    },

    getAuthenticatedEntity() {
        requireCorrelation();

        return domain.active[httpInfraSymbol].authenticatedEntity;
    },

    isAuthenticatedEntity() {
        return !(_.isUndefined(domain.active)) &&
            !(_.isNull(domain.active[httpInfraSymbol].authenticatedEntity));
    },

    setAuthenticatedEntity(authenticatedEntity, allowOverride) {
        const httpInfraData = domain.active[httpInfraSymbol];

        if (!httpInfraData) {
            throw new Error('Can\'t set authenticated entity. Domain isn\'t registered.');
        }

        if (!allowOverride && httpInfraData.authenticatedEntity) {
            throw new Error('Can\'t set authenticated entity. Another entity is already set.');
        }

        httpInfraData.authenticatedEntity = authenticatedEntity;
    },
};
